# -*- coding: UTF-8 -*-
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

import pandas as pd
import copy
from uqer.SignalAnalysis.const import RISK_INDUSTRY_MAP, INDUSTRY_VERSION_MAP, INDUSTRY_SW14_TO_21, \
    RM_STYLE_MAPPING


REFRESH_RATE_LIST = ['fri', 'month']
REFRESH_RATE_MAP = {}


def _validate_field(field, valid_fields):
    return field in valid_fields


def list_type_check(data_list, type_tuple):
    res = True
    for data in data_list:
        if not isinstance(data, type_tuple):
            res = False
    return res


def print_validate_risk_aversion(risk_aversion):
    if isinstance(risk_aversion,(int,float)) and risk_aversion >= 0:
        return True
    else:
        print("risk_aversion必须为大于等于0的float/int")
        return False


def print_validate_soft_params(soft_params):
    style_factors = ['factor_exposure_BETA', 'factor_exposure_BTOP', 'factor_exposure_EARNYILD',
                     'factor_exposure_GROWTH',
                     'factor_exposure_LEVERAGE', 'factor_exposure_LIQUIDTY', 'factor_exposure_MOMENTUM',
                     'factor_exposure_RESVOL',
                     'factor_exposure_SIZE', 'factor_exposure_SIZENL', 'factor_exposure_LightIndus',
                     'factor_exposure_HouseApp',
                     'factor_exposure_NonFerMetal', 'factor_exposure_Media', 'factor_exposure_CONMAT',
                     'factor_exposure_Textile',
                     'factor_exposure_ELECEQP', 'factor_exposure_Electronics', 'factor_exposure_CHEM',
                     'factor_exposure_Mining',
                     'factor_exposure_Transportation', 'factor_exposure_NonBankFinan', 'factor_exposure_Auto',
                     'factor_exposure_CommeTrade', 'factor_exposure_Utilities', 'factor_exposure_FoodBever',
                     'factor_exposure_AgriForest', 'factor_exposure_Conglomerates', 'factor_exposure_IronSteel',
                     'factor_exposure_MachiEquip', 'factor_exposure_Telecom', 'factor_exposure_Computer',
                     'factor_exposure_RealEstate', 'factor_exposure_Health', 'factor_exposure_AERODEF',
                     'factor_exposure_LeiService', 'factor_exposure_BuildDeco', 'factor_exposure_Bank']
    validation_result = False
    if isinstance(soft_params, pd.DataFrame):
        if soft_params.empty:
            validation_result = True
        elif ['penalty', 'priority'] == list(soft_params.columns) or ['priority', 'penalty'] == list(
                soft_params.columns):
            indexes = list(soft_params.index)
            if set(indexes).issubset(
                    set(['target_risk_limit', 'asset_bound_limit', 'target_turnover_limit'] + style_factors)):
                if len(soft_params) > len(set(soft_params['priority'])):
                    print('soft_params的priority错误,priority不能有重复值')
                else:
                    if list_type_check(soft_params['priority'], (int, float)) and list_type_check(
                            soft_params['penalty'], (int, float)):
                        validation_result = True
                    else:
                        print("priority和penalty的数据类型错误，正确类型为int或float")
            else:
                print("soft_params的index错误,index在以下四个枚举值中：target_risk_limit,asset_bound_limit,factor_exposure_风格名,target_turnover_limit")
        else:
            print("soft_params的column错误,soft_params.columns为['priority','penalty']")
    else:
        print('soft_params数据类型错误,请传入pd.Dataframe类型')
    return validation_result


def print_validate_frequency(freq):
    validation_result = _validate_field(
        freq, ['day', 'week', 'month', 'quarter', 'semi-year'])
    if not validation_result:
        print("frequency参数：%s暂不支持，支持范围：[day/week/month/quarter/semi-year]" % freq)
    return validation_result


def print_validate_universe(univ):
    all_benchmark_list = ['HS300', 'ZZ500', 'ZZ700', 'ZZ800', 'TLQA', 'SH50', 'SC50', 'ZZ1000', 'ZZQZ']
    validation_result = _validate_field(
        univ, all_benchmark_list)
    custom_universe = isinstance(univ, dict)
    validation_result = validation_result or custom_universe
    if not validation_result:
        print('''universe参数：%s暂不支持，支持范围： %s或者自定义universe''' % (univ, '/'.join(all_benchmark_list)))
    return validation_result


def print_validate_benchmark(benchmark):
    all_benchmark_list = ['HS300', 'ZZ500', 'ZZ700', 'ZZ800', 'TLQA', 'risk_free', 'SH50', 'SC50', 'ZZ1000', 'ZZQZ']
    validation_result = benchmark is None or _validate_field(
        benchmark, all_benchmark_list)
    custom_benchmark = isinstance(benchmark, dict)
    validation_result = validation_result or custom_benchmark
    if not validation_result:
        print('''benchmark参数：%s暂不支持，支持范围：None值或者 %s或者自定义benchmark''' % (benchmark, '/'.join(all_benchmark_list)))
    return validation_result


def print_validate_weight_type(weight_type):
    validation_result = _validate_field(weight_type, ['equal', 'cap', 'risk'])
    if not validation_result:
        print("weight_type参数：%s暂不支持，支持范围：equal/cap/risk" % weight_type)
    return validation_result


def print_validate_select_type(select_type):
    validation_result = _validate_field(select_type, [0, 1])
    if not validation_result:
        print("select_type参数：%s暂不支持，支持范围：0/1" % select_type)
    return validation_result


def print_validate_construct_method(construct_method):
    validation_result = _validate_field(construct_method,
                                        ['simple_long_only',
                                         'simple_long_short',
                                         'ideal', 'limit_active_risk',
                                         'limit_portfolio_risk',
                                         'max_sharpe_ratio'])
    if not validation_result:
        print('''select_type参数：%s暂不支持，支持范围：simple_long_only/
        simple_long_short/ideal/limit_active_risk/limit_portfolio_risk/
        max_sharpe_ratio''' % construct_method)
    return validation_result

def print_valid_sector_exposure(sector_exposure_boundary, risk_model_version):
    """
    校验 sector_exposure输入行业
    Args:
        sector_exposure_boundary
        risk_model_version: str
    Returns

    """
    invalid_industry_list = []
    industry_version = RISK_INDUSTRY_MAP.get(risk_model_version)
    if risk_model_version == 'dy_cne5_old':
        industry_limit_list = list((set(INDUSTRY_VERSION_MAP[industry_version]) - set(list(RM_STYLE_MAPPING.keys()))) | \
                              set(list(INDUSTRY_SW14_TO_21.values())))
    elif risk_model_version == 'dy_cne5_new':
        industry_limit_list = list((set(INDUSTRY_VERSION_MAP[industry_version]) - set(list(RM_STYLE_MAPPING.keys()))) | \
                              set(list(INDUSTRY_SW14_TO_21.keys())))
    if isinstance(sector_exposure_boundary, dict):
        invalid_industry_list = list(set(sector_exposure_boundary.keys()) - set(industry_limit_list))
    elif isinstance(sector_exposure_boundary, pd.Series):
        invalid_industry_list = list(set(sector_exposure_boundary.index) - set(industry_limit_list))

    if invalid_industry_list:
        print('选择"{}"风险模型时，{}行业不是可支持输入的行业，请浏览帮助文档查询可输入的行业'.format(risk_model_version,
                                                                  "/".join(invalid_industry_list)))
        return False
    return True


def replace_industry_name(sector_exposure_boundary, risk_model_version):
    """
    替换 sector_exposure SW21/Sw14行业
    Args:
        sector_exposure_boundary
        risk_model_version: str
    Returns
    """
    INDUSTRY_SW21_TO_14 = {v: k for k, v in INDUSTRY_SW14_TO_21.items()}
    industry_name_mapping = INDUSTRY_SW21_TO_14 if risk_model_version == 'dy_cne5_old' else INDUSTRY_SW14_TO_21
    sector_copy = copy.deepcopy(sector_exposure_boundary)
    if isinstance(sector_copy, dict):
        sector_copy = {industry_name_mapping.get(k, k): v for k, v in sector_copy.items()}
    elif isinstance(sector_copy, pd.Series):
        sector_copy.index = [industry_name_mapping.get(k, k) for k in sector_copy.index]
    return sector_copy


# -*- coding: utf-8 -*-
# 通联数据机密
# --------------------------------------------------------------------
# 通联数据股份公司版权所有 © 2013-2022
#
# 注意：本文所载所有信息均属于通联数据股份公司资产。本文所包含的知识和技术概念均属于
# 通联数据产权，并可能由中国、美国和其他国家专利或申请中的专利所覆盖，并受商业秘密或
# 版权法保护。
# 除非事先获得通联数据股份公司书面许可，严禁传播文中信息或复制本材料。
#
# DataYes CONFIDENTIAL
# --------------------------------------------------------------------
# Copyright © 2013-2022 DataYes, All Rights Reserved.
#
# NOTICE: All information contained herein is the property of DataYes
# Incorporated. The intellectual and technical concepts contained herein are
# proprietary to DataYes Incorporated, and may be covered by China, U.S. and
# Other Countries Patents, patents in process, and are protected by trade
# secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from DataYes.
from __future__ import unicode_literals
from __future__ import print_function

import sys
import requests
import traceback
import os

from .config import *
from .utils import format_print


class Authorize(object):
    """优矿
    """
    def __init__(self, token='', session=None):
        self.__token = token
        self.session = session


    def _authorize(self):
        """账户验证
        """
        token = self.__token
        if token:
            try:
                self._isvalid, is_network_ok = self.__is_token_valid(token)
            except SystemExit:
                raise
            if not is_network_ok:
                format_print('网络异常，无法验证token，请检查当前网络连通性', with_date=True)
            elif not self._isvalid:
                format_print('抱歉，您的 token 验证失败：{}'.format(token), with_date=True)
            else:
                token = self.__get_permanent_token_and_set_to_cookie(token=token)
                username = os.environ.get('DatayesPrincipalName', 'unknow')
                format_print('{} 账号登录成功'.format(username))


    def __get_permanent_token_and_set_to_cookie(self, token='', cookies={}):
        if not token:
            ret_json = requests.post(TOKEN_URL, data={'grant_type':'permanent'}, cookies=cookies).json()
            token = ret_json.get('access_token')

        self.__set_token_to_cookie(token)

        return token


    def __set_token_to_cookie(self, token):

        os.environ['access_token'] = token

        cookie_dict = {'cloud-sso-token': token}
        self.session.cookies = requests.utils.cookiejar_from_dict(cookie_dict)
        return token


    # def __authorize_user(self, user, pwd):
    #
    #     ### 2 user type
    #     data_type = dict(username=user, password=pwd, app='mercury_sdk')
    #
    #     def user_type(data=None):
    #         res = self.session.post(AUTHORIZE_URL, data)
    #
    #         if not res.ok or not res.json().get('content', {}).get('accountId', 0):
    #             return False, None
    #         else:
    #             result = res.json()
    #             token = result.get('content', {}).get('token', {}).get('tokenString', '')
    #             principal_name = result.get('content', {}).get('principalName', '')
    #             os.environ['DatayesPrincipalName'] = principal_name
    #             return True, token
    #
    #     valid, token = user_type(data_type)
    #
    #     if not valid:
    #         return False, None
    #     else:
    #         os.environ['cloud_sso_token'] = token
    #         return True, token


    def __is_token_valid(self, token):
        """
        检验 token 是否有效
        Args:
             token: str
                用户token
        Returns: tuple
                 is_token_valid, is_network_ok
        """
        try:
            r = None
            r = self.session.get(PROFILE_URL, cookies={'cloud-sso-token': token})
            r_json = r.json()

            if type(r_json) == dict and r_json.get('code', 0) == 200:
                r = self.session.get(UQER_AUTH_URL, cookies={'cloud-sso-token': token})
                r_json = r.json()

                os.environ['DatayesPrincipalName'] = r_json['user']['principalName']
                # 校验用户是否有uqer.sdk.data的权限，若没有，则直接退出
                url = OPS_URL
                res = self.session.get(url, headers={'DatayesPrincipalName': os.environ['DatayesPrincipalName']},
                                       cookies={'cloud-sso-token': token})
                r_json = res.json()
                if r_json['code'] != 200:
                    format_print('获取用户权限数据异常', with_date=True)
                    return False, True

                if r_json['data'].get('uqer.sdk.data') is True:
                    for key, value in r_json['data'].items():
                        os.environ[key] = '1' if value is True else '0'
                    return True, True

                format_print('无UQER SDK权限，可以联系4000 820 386进行购买。')
                sys.exit(0)
            elif type(r_json) == dict and r_json.get('code', 0) == -403:
                format_print('token {} 无效或过期'.format(token), with_date=True)
                return False, True
            else:
                format_print('token 验证异常: {}'.format(r.text), with_date=True)
                return False, True
        except SystemExit:
            raise
        except:
            format_print('网络异常, 无法验证token', with_date=True)
            format_print('-' * 80)
            if r:
                format_print('Check token failed: url is %s, http code is %s, data is %s' % (PROFILE_URL, r.status_code, r.text), with_date=True)
            else:
                format_print('Check token failed: url is %s' % (PROFILE_URL), with_date=True)
            traceback.print_exc()
            format_print('-' * 80)
            return False, False












# -*- coding: utf-8 -*-
# 通联数据机密
# --------------------------------------------------------------------
# 通联数据股份公司版权所有 © 2013-2025
#
# 注意：本文所载所有信息均属于通联数据股份公司资产。本文所包含的知识和技术概念均属于
# 通联数据产权，并可能由中国、美国和其他国家专利或申请中的专利所覆盖，并受商业秘密或
# 版权法保护。
# 除非事先获得通联数据股份公司书面许可，严禁传播文中信息或复制本材料。
#
# DataYes CONFIDENTIAL
# --------------------------------------------------------------------
# Copyright © 2013-2025 DataYes, All Rights Reserved.
#
# NOTICE: All information contained herein is the property of DataYes
# Incorporated. The intellectual and technical concepts contained herein are
# proprietary to DataYes Incorporated, and may be covered by China, U.S. and
# Other Countries Patents, patents in process, and are protected by trade
# secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from DataYes.
import sys
import os
import requests
from pip._vendor.packaging import version as packaging_version
from pip._vendor import pkg_resources
try:
    from pip._vendor.six.moves.urllib import parse as urllib_parse
except ImportError:
    import urllib.parse as urllib_parse

WINDOWS = (sys.platform.startswith("win") or (sys.platform == 'cli' and os.name == 'nt'))


class Index(object):
    def __init__(self, url):
        self.url = url
        self.netloc = urllib_parse.urlsplit(url).netloc
        self.simple_url = self.url_to_path('simple')
        self.pypi_url = self.url_to_path('pypi')
        self.pip_json_url = self.url_to_path('pypi/pip/json')

    def url_to_path(self, path):
        return urllib_parse.urljoin(self.url, path)


PyPI = Index('https://pypi.org/')


def get_installed_version(dist_name, lookup_dirs=None):
    """Get the installed version of dist_name avoiding pkg_resources cache"""
    # Create a requirement that we'll look for inside of setuptools.
    req = pkg_resources.Requirement.parse(dist_name)

    # We want to avoid having this cached, so we need to construct a new
    # working set each time.
    if lookup_dirs is None:
        working_set = pkg_resources.WorkingSet()
    else:
        working_set = pkg_resources.WorkingSet(lookup_dirs)

    # Get the installed distribution from our working set
    dist = working_set.find(req)

    # Check to see if we got an installed distribution or not, if we did
    # we want to return it's version.
    return dist.version if dist else ""


def check_uqer_version():
    installed_version = get_installed_version('uqer')
    pip_version = packaging_version.parse(installed_version)
    try:
        resp = requests.get(
            PyPI.url_to_path('pypi/uqer/json'),
            headers={"Accept": "application/json"},
            timeout=5
        )
        resp.raise_for_status()
        pypi_version = [
            v for v in sorted(
                list(resp.json()["releases"]),
                key=packaging_version.parse,
            )
            if not packaging_version.parse(v).is_prerelease
        ][-1]
    except Exception:
        pass
    else:
        remote_version = packaging_version.parse(pypi_version)
        pip_main_version = int(pip_version.base_version.split('.')[0])
        remote_main_version = int(remote_version.base_version.split('.')[0])
        if pip_main_version < remote_main_version:
            pip_cmd = "python -m pip" if WINDOWS else "pip"
            raise Exception(
                "你使用的uqer版本是%s, 然而%s已经可以使用。\n"
                "你必须使用'%s install --upgrade uqer'命令进行升级。" % (pip_version, remote_version, pip_cmd)
                )


# -*- coding: utf-8 -*-
# 通联数据机密
# --------------------------------------------------------------------
# 通联数据股份公司版权所有 © 2013-2022
#
# 注意：本文所载所有信息均属于通联数据股份公司资产。本文所包含的知识和技术概念均属于
# 通联数据产权，并可能由中国、美国和其他国家专利或申请中的专利所覆盖，并受商业秘密或
# 版权法保护。
# 除非事先获得通联数据股份公司书面许可，严禁传播文中信息或复制本材料。
#
# DataYes CONFIDENTIAL
# --------------------------------------------------------------------
# Copyright © 2013-2022 DataYes, All Rights Reserved.
#
# NOTICE: All information contained herein is the property of DataYes
# Incorporated. The intellectual and technical concepts contained herein are
# proprietary to DataYes Incorporated, and may be covered by China, U.S. and
# Other Countries Patents, patents in process, and are protected by trade
# secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from DataYes.

__all__ = ['ideal', 'long_only', 'long_short', 'limit_active_risk', 'limit_portfolio_risk']

import pandas as pd

from .SignalAnalysis.tears import portfolio_construction
from .utils import Authorization

_OPTIMIZER_RETURN_FIELDS = [
    'trade_volume',
    'trade_value',
    'expected_volume',
    'expected_active_weight',
    'expected_weight',
    'expected_holding',
    'opt_status',
    'soft_cons'
]
_SIMPLE_RETURN_FIELDS = [
    'expected_weight',
]


def print_validate_factor_frame(factor_frame):
    is_all_null = factor_frame.isnull().all().all()
    if is_all_null:
        print("因子数据异常，请校验")
    return not is_all_null


@Authorization('uqer.py.PortfolioConstruction')
def ideal(
        factor_frame, start_date, end_date, freq, universe='HS300', benchmark='HS300',
        init_cash=1e8, init_holding=None, risk_model_version='dy_cne5_old'
):
    """
    :param factor_frame: 因子数据, DataFrame(factor_values, index=dates, columns=tickers)
    :param start_date: str, 'YYYYmmdd'
    :param end_date: str, 'YYYYmmdd'
    :param freq: str, 调仓频率，'day/week/month/quarter/semi-year'
    :param universe: str or dict of list, 股票池。默认是HS300，目前支持HS300/ZZ500/ZZ700/ZZ800/TLQA/SH50/SC50/ZZ1000/ZZQZ,
            也可为dict of list，则为自定义universe
    :param benchmark: str/dict of Series/dict of dict, 优化基准。
                      支持指数成份股 HS300/ZZ500/ZZ700/ZZ800/TLQA/risk_free/SH50/SC50/ZZ1000/ZZQZ。默认为HS300
                      支持自定义指数,即dict of Series/dict of dict形式定义的benchmark
    :param init_cash: int/float, 初始资金，默认1e8
    :param init_holding: dict/pd.Series/None, 每个个股的初始市值，不包含现金
    :param risk_model_version: str, 选择不同的风险模型作为组合优化的约束依据
    :return: dict
    """
    if not print_validate_factor_frame(factor_frame):
        return
    if risk_model_version not in ['dy_cne5_old', 'dy_cne5_new']:
        raise Exception("risk_model_version仅支持dy_cne5_old/dy_cne5_new")

    tickers = factor_frame.columns.values
    if len(tickers[0]) > 6:
        tickers_new = [ticker[:6] for ticker in tickers]
        factor_frame.columns = tickers_new

    # 宽格式转换为长格式
    factor_value_frame = factor_frame.copy()
    factor_value_frame.index.name = 'date'
    factor_value_frame.reset_index(inplace=True)
    factor_value_frame = pd.melt(factor_value_frame, id_vars=['date'], var_name='ticker', value_name='value')
    factor_value_frame['date'] = factor_value_frame['date'].apply(lambda x: x.replace('-', ''))

    result = portfolio_construction(
        factor_value_frame=factor_value_frame,
        construct_method='ideal',
        start_date=start_date,
        end_date=end_date,
        frequency=freq,
        universe=universe,
        benchmark=benchmark,
        init_cash=init_cash,
        simulate=True,
        init_holding=init_holding
    )
    if result:
        result = {
            "-".join([td[:4], td[4:6], td[6:8]]): {f: result['trades'][td].get(f) for f in _OPTIMIZER_RETURN_FIELDS}
            for td in result['trades']
        }
    return result


@Authorization('uqer.py.PortfolioConstruction')
def long_only(
        factor_frame, start_date, end_date, freq, universe='HS300', benchmark='HS300',
        top_ratio=0.2, select_type=1, weight_type='equal', init_cash=1e8, init_holding=None,
):
    """
    :param factor_frame: 因子数据, DataFrame(factor_values, index=dates, columns=tickers)
    :param start_date: str, 'YYYYmmdd'
    :param end_date: str, 'YYYYmmdd'
    :param freq: str, 调仓频率，'day/week/month/quarter/semi-year'
    :param universe: str or dict, 股票池。默认是HS300，目前支持HS300/ZZ500/ZZ700/ZZ800/TLQA/SH50/SC50/ZZ1000/ZZQZ,
            也可为dict of list，则为自定义universe
    :param benchmark: str/dict of Series/dict of dict, 优化基准。
                      支持指数成份股 HS300/ZZ500/ZZ700/ZZ800/TLQA/risk_free/SH50/SC50/ZZ1000/ZZQZ。默认为HS300
                      支持自定义指数,即dict of Series/dict of dict形式定义的benchmark
    :param top_ratio: float/list 做多前多少百分比或者前百分比区间的股票，默认0.2
    :param select_type: int，组合构建的行业配比方式，默认为1。可选0/1 WHOLE_INDUSTRY = 0 EACH_INDUSTRY = 1
    :param weight_type: str, 权重配比方式，取值范围equal/risk/cap，对应为等权/风险权重/流通市值权重，默认equal
    :param init_cash: int/float, 初始资金，默认1e8
    :param init_holding: dict/pd.Series/None, 每个个股的初始市值，不包含现金
    :return: dict
    """
    if not print_validate_factor_frame(factor_frame):
        return

    tickers = factor_frame.columns.values
    if len(tickers[0]) > 6:
        tickers_new = [ticker[:6] for ticker in tickers]
        factor_frame.columns = tickers_new

    # 宽格式转换为长格式
    factor_value_frame = factor_frame.copy()
    factor_value_frame.index.name = 'date'
    factor_value_frame.reset_index(inplace=True)
    factor_value_frame = pd.melt(factor_value_frame, id_vars=['date'], var_name='ticker', value_name='value')
    factor_value_frame['date'] = factor_value_frame['date'].apply(lambda x: x.replace('-', ''))

    result = portfolio_construction(
        factor_value_frame=factor_value_frame,
        construct_method='simple_long_only',
        start_date=start_date,
        end_date=end_date,
        frequency=freq,
        universe=universe,
        benchmark=benchmark,
        top_ratio=top_ratio,
        select_type=select_type,
        weight_type=weight_type,
        init_cash=init_cash,
        init_holding=init_holding,
        simulate=True
    )
    if result:
        result = {
            "-".join([td[:4], td[4:6], td[6:8]]): {f: result['trades'][td].get(f) for f in _OPTIMIZER_RETURN_FIELDS}
            for td in result['trades']
        }
    return result


@Authorization('uqer.py.PortfolioConstruction')
def long_short(
        factor_frame, start_date, end_date, freq, universe='HS300', benchmark='HS300',
        top_ratio=0.2, down_ratio=0.2, select_type=1, weight_type='equal', init_cash=1e8, init_holding=None,
):
    """
    :param factor_frame: 因子数据, DataFrame(factor_values, index=dates, columns=tickers)
    :param start_date: str, 'YYYYmmdd'
    :param end_date: str, 'YYYYmmdd'
    :param freq: str, 调仓频率，'day/week/month/quarter/semi-year'
    :param universe: str or dict, 股票池。默认是HS300，目前支持HS300/ZZ500/ZZ700/ZZ800/TLQA/SH50/SC50/ZZ1000/ZZQZ,
            也可为dict of list，则为自定义universe
    :param benchmark: str/dict of Series/dict of dict, 优化基准。
                      支持指数成份股 HS300/ZZ500/ZZ700/ZZ800/TLQA/risk_free/SH50/SC50/ZZ1000/ZZQZ。默认为HS300
                      支持自定义指数,即dict of Series/dict of dict形式定义的benchmark
    :param top_ratio: float/list 做多前多少百分比或者前百分比区间的股票，默认0.2
    :param down_ratio: float 做空后多少百分比的股票，默认0.2
    :param select_type: int，组合构建的行业配比方式，默认为1。可选0/1 WHOLE_INDUSTRY = 0 EACH_INDUSTRY = 1
    :param weight_type: str, 权重配比方式，取值范围equal/risk/cap，对应为等权/风险权重/流通市值权重，默认equal
    :param init_cash: int/float, 初始资金，默认1e8
    :param init_holding: dict/pd.Series/None, 每个个股的初始市值，不包含现金
    :return: dict
    """
    if not print_validate_factor_frame(factor_frame):
        return

    tickers = factor_frame.columns.values
    if len(tickers[0]) > 6:
        tickers_new = [ticker[:6] for ticker in tickers]
        factor_frame.columns = tickers_new

    # 宽格式转换为长格式
    factor_value_frame = factor_frame.copy()
    factor_value_frame.index.name = 'date'
    factor_value_frame.reset_index(inplace=True)
    factor_value_frame = pd.melt(factor_value_frame, id_vars=['date'], var_name='ticker', value_name='value')
    factor_value_frame['date'] = factor_value_frame['date'].apply(lambda x: x.replace('-', ''))

    result = portfolio_construction(
        factor_value_frame=factor_value_frame,
        construct_method='simple_long_short',
        start_date=start_date,
        end_date=end_date,
        frequency=freq,
        universe=universe,
        benchmark=benchmark,
        top_ratio=top_ratio,
        down_ratio=down_ratio,
        select_type=select_type,
        weight_type=weight_type,
        init_cash=init_cash,
        init_holding=init_holding,
    )
    if result:
        result = {
            "-".join([td[:4], td[4:6], td[6:8]]): {f: result[td].get(f) for f in _OPTIMIZER_RETURN_FIELDS}
            for td in result
        }
    return result


@Authorization('uqer.py.PortfolioConstruction')
def limit_active_risk(
        factor_frame, start_date, end_date, freq, universe='HS300', benchmark='HS300', price_type='open',
        target_risk=0.3, target_turnover=None, fee=0, init_cash=1e8, init_holding=None,
        asset_lower_boundary=0, asset_upper_boundary=1,
        asset_cons_method='absolute', asset_cons_type='by_value',
        factor_exposure_lower_boundary=-10, factor_exposure_upper_boundary=10,
        sector_exposure_lower_boundary=-10, sector_exposure_upper_boundary=10,
        risk_aversion=0, soft_params=None, cash_boundary=None, risk_model_version='dy_cne5_old',
        asset_group_lower_boundary={}, asset_group_upper_boundary={}
):
    """
    :param factor_frame:
    :param start_date: str, 'YYYYmmdd'
    :param end_date: str, 'YYYYmmdd'
    :param freq: str, 调仓频率，'day/week/month/quarter/semi-year'
    :param universe: str or dict, 股票池。默认是HS300，目前支持HS300/ZZ500/ZZ700/ZZ800/TLQA/SH50/SC50/ZZ1000/ZZQZ,
            也可为dict of list，则为自定义universe
    :param benchmark: str/dict of Series/dict of dict, 优化基准。
                      支持指数成份股 HS300/ZZ500/ZZ700/ZZ800/TLQA/risk_free/SH50/SC50/ZZ1000/ZZQZ。默认为HS300
                      支持自定义指数,即dict of Series/dict of dict形式定义的benchmark
    :param price_type: str, 估值价格，取值范围open/close,默认open.
            open 为使用开盘价进行组合估值，close 为使用收盘价进行组合估值。
    :param target_risk: float, 默认值0.3
    :param target_turnover: float/int/None, 目标换手率约束，默认值为None
    :param fee: float/int, 交易费用线性系数
    :param init_cash: int/float, 初始资金，默认1e8
    :param init_holding: dict/pd.Series/None, 每个个股的初始市值，不包含现金
    :param asset_lower_boundary: float/Series，个股仓位下限，默认为0
    :param asset_upper_boundary: float/Series，个股仓位上限，默认为1
    :param asset_cons_method: str， 资产上下界约束方法，支持选择'absolute'/'active'：绝对权重约束/主动权重约束
    :param asset_cons_type: str， 资产主动权重约束类型，支持选择'by_value'/'by_percent'：具体值主动约束/百分比主动约束
    :param factor_exposure_lower_boundary: float/Series，风格因子暴露下限，默认为-10
    :param factor_exposure_upper_boundary: float/Series，风格因子暴露上限，默认为10
    :param sector_exposure_lower_boundary: float/Series，行业因子暴露下限，默认为-10
    :param sector_exposure_upper_boundary: float/Series，行业因子暴露上限，默认为10
    :param risk_aversion: float,风险厌恶系数, >= 0，如果不传则默认为0
    :param soft_params: pd.DataFrame, 软约束参数
    :param cash_boundary: list of float, 现金约束,默认[0.0,0.01],即默认为满仓约束,如果传[0.05,0.1],说明现金约束在5%～10%之间
    :param asset_group_lower_boundary: 分组资产权重总和约束下界, dict, key为调仓日期，value为list of Series
    :param asset_group_upper_boundary: 分组资产权重总和约束上界, dict, key为调仓日期，value为list of Series
    :param risk_model_version: str, 可以选择不同的风险模型作为组合优化的约束依据
    :return: dict
    """
    if soft_params is None:
        soft_params = pd.DataFrame()
    if cash_boundary is None:
        cash_boundary = [0.0, 0.01]
    if not print_validate_factor_frame(factor_frame):
        return

    tickers = factor_frame.columns.values
    if len(tickers[0]) > 6:
        tickers_new = [ticker[:6] for ticker in tickers]
        factor_frame.columns = tickers_new

    # 宽格式转换为长格式
    factor_value_frame = factor_frame.copy()
    factor_value_frame.index.name = 'date'
    factor_value_frame.reset_index(inplace=True)
    factor_value_frame = pd.melt(factor_value_frame, id_vars=['date'], var_name='ticker', value_name='value')
    factor_value_frame['date'] = factor_value_frame['date'].apply(lambda x: x.replace('-', ''))
    if asset_cons_method not in ['absolute', 'active']:
        raise Exception('asset_cons_method not in ["absolute","active"]')
    if asset_cons_type not in ['by_value', 'by_percent']:
        raise Exception('asset_cons_type not in ["by_value","by_percent"]')
    if risk_model_version not in ['dy_cne5_old', 'dy_cne5_new']:
        raise Exception("risk_model_version仅支持dy_cne5_old/dy_cne5_new")

    result = portfolio_construction(
        factor_value_frame=factor_value_frame,
        construct_method='limit_active_risk',
        start_date=start_date,
        end_date=end_date,
        frequency=freq,
        universe=universe,
        benchmark=benchmark,
        price_type=price_type,
        target_risk=target_risk,
        target_turnover=target_turnover,
        fee=fee,
        init_cash=init_cash,
        init_holding=init_holding,
        asset_lower_boundary=asset_lower_boundary,
        asset_upper_boundary=asset_upper_boundary,
        asset_cons_method=asset_cons_method,
        asset_cons_type=asset_cons_type,
        factor_exposure_lower_boundary=factor_exposure_lower_boundary,
        factor_exposure_upper_boundary=factor_exposure_upper_boundary,
        sector_exposure_lower_boundary=sector_exposure_lower_boundary,
        sector_exposure_upper_boundary=sector_exposure_upper_boundary,
        asset_group_lower_boundary=asset_group_lower_boundary,
        asset_group_upper_boundary=asset_group_upper_boundary,
        risk_aversion=risk_aversion,
        soft_params=soft_params,
        cash_boundary=cash_boundary,
        risk_model_version=risk_model_version,
    )
    if result:
        result = {
            "-".join([td[:4], td[4:6], td[6:8]]): {f: result[td].get(f) for f in _OPTIMIZER_RETURN_FIELDS}
            for td in result
        }
    return result


@Authorization('uqer.py.PortfolioConstruction')
def limit_portfolio_risk(
        factor_frame, start_date, end_date, freq, universe='HS300', benchmark='HS300', price_type='open',
        target_risk=0.3, target_turnover=None, fee=0, init_cash=1e8, init_holding=None,
        asset_lower_boundary=0, asset_upper_boundary=1,
        asset_cons_method='absolute', asset_cons_type='by_value',
        factor_exposure_lower_boundary=-10, factor_exposure_upper_boundary=10,
        sector_exposure_lower_boundary=-10, sector_exposure_upper_boundary=10,
        risk_aversion=0, soft_params=None, cash_boundary=None, risk_model_version='dy_cne5_old',
        asset_group_lower_boundary={}, asset_group_upper_boundary={}
):
    """
    :param factor_frame:
    :param start_date: str, 'YYYYmmdd'
    :param end_date: str, 'YYYYmmdd'
    :param freq: str, 调仓频率，'day/week/month/quarter/semi-year'
    :param universe: str or dict, 股票池。默认是HS300，目前支持HS300/ZZ500/ZZ700/ZZ800/TLQA/SH50/SC50/ZZ1000/ZZQZ,
            也可为dict of list，则为自定义universe
    :param benchmark: str/dict of Series/dict of dict, 优化基准。
                      支持指数成份股 HS300/ZZ500/ZZ700/ZZ800/TLQA/risk_free/SH50/SC50/ZZ1000/ZZQZ。默认为HS300
                      支持自定义指数,即dict of Series/dict of dict形式定义的benchmark
    :param price_type: str, 估值价格，取值范围open/close,默认open.
            open 为使用开盘价进行组合估值，close 为使用收盘价进行组合估值。
    :param target_risk: float, 默认值0.3
    :param target_turnover: float/int/None, 目标换手率约束，默认值为None
    :param fee: float/int, 交易费用线性系数
    :param init_cash: int/float, 初始资金，默认1e8
    :param init_holding: dict/pd.Series/None, 每个个股的初始市值，不包含现金
    :param asset_lower_boundary: float/Series，个股仓位下限，默认为0
    :param asset_upper_boundary: float/Series，个股仓位上限，默认为1
    :param asset_cons_method: str， 资产上下界约束方法，支持选择'absolute'/'active'：绝对权重约束/主动权重约束
    :param asset_cons_type: str， 资产主动权重约束类型，支持选择'by_value'/'by_percent'：具体值主动约束/百分比主动约束
    :param factor_exposure_lower_boundary: float/Series，风格因子暴露下限，默认为-10
    :param factor_exposure_upper_boundary: float/Series，风格因子暴露上限，默认为10
    :param sector_exposure_lower_boundary: float/Series，行业因子暴露下限，默认为-10
    :param sector_exposure_upper_boundary: float/Series，行业因子暴露上限，默认为10
    :param risk_aversion: float,风险厌恶系数, >= 0，如果不传则默认为0
    :param soft_params: pd.DataFrame, 软约束参数
    :param cash_boundary: list of float, 现金约束,默认[0.0,0.01],即默认为满仓约束,如果传[0.05,0.1],说明现金约束在5%～10%之间
    :param asset_group_lower_boundary: 分组资产权重总和约束下界, dict, key为调仓日期，value为list of Series
    :param asset_group_upper_boundary: 分组资产权重总和约束上界, dict, key为调仓日期，value为list of Series
    :param risk_model_version: str, 可以选择不同的风险模型作为组合优化的约束依据
    :return: dict
    """
    if soft_params is None:
        soft_params = pd.DataFrame()
    if cash_boundary is None:
        cash_boundary = [0.0, 0.01]
    if not print_validate_factor_frame(factor_frame):
        return

    tickers = factor_frame.columns.values
    if len(tickers[0]) > 6:
        tickers_new = [ticker[:6] for ticker in tickers]
        factor_frame.columns = tickers_new

    # 宽格式转换为长格式
    factor_value_frame = factor_frame.copy()
    factor_value_frame.index.name = 'date'
    factor_value_frame.reset_index(inplace=True)
    factor_value_frame = pd.melt(factor_value_frame, id_vars=['date'], var_name='ticker', value_name='value')
    factor_value_frame['date'] = factor_value_frame['date'].apply(lambda x: x.replace('-', ''))

    if asset_cons_method not in ['absolute', 'active']:
        raise Exception('asset_cons_method not in ["absolute","active"]')
    if asset_cons_type not in ['by_value', 'by_percent']:
        raise Exception('asset_cons_type not in ["by_value","by_percent"]')
    if risk_model_version not in ['dy_cne5_old', 'dy_cne5_new']:
        raise Exception("risk_model_version仅支持dy_cne5_old/dy_cne5_new")

    result = portfolio_construction(
        factor_value_frame=factor_value_frame,
        construct_method='limit_portfolio_risk',
        start_date=start_date,
        end_date=end_date,
        frequency=freq,
        universe=universe,
        benchmark=benchmark,
        price_type=price_type,
        target_risk=target_risk,
        target_turnover=target_turnover,
        fee=fee,
        init_cash=init_cash,
        init_holding=init_holding,
        asset_lower_boundary=asset_lower_boundary,
        asset_upper_boundary=asset_upper_boundary,
        asset_cons_method=asset_cons_method,
        asset_cons_type=asset_cons_type,
        factor_exposure_lower_boundary=factor_exposure_lower_boundary,
        factor_exposure_upper_boundary=factor_exposure_upper_boundary,
        sector_exposure_lower_boundary=sector_exposure_lower_boundary,
        sector_exposure_upper_boundary=sector_exposure_upper_boundary,
        asset_group_lower_boundary=asset_group_lower_boundary,
        asset_group_upper_boundary=asset_group_upper_boundary,
        risk_aversion=risk_aversion,
        soft_params=soft_params,
        cash_boundary=cash_boundary
    )
    if result:
        result = {
            "-".join([td[:4], td[4:6], td[6:8]]): {f: result[td].get(f) for f in _OPTIMIZER_RETURN_FIELDS}
            for td in result
        }
    return result



# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals


import os
import sys
import requests
from requests import utils
from .version import __version__
utils.default_user_agent = lambda: 'mercury-sdk/%s' % __version__

from . import uqer
from .uqer import Client
from .mfclient import neutralize, standardize, winsorize, simple_long_only, long_only, neutralize_pit
from . import DataAPI

from .utils import format_print

from .DataAPI import retry_interval, max_retries
from .DataAPI.api_base import __getConn__
from .upgrade import check_uqer_version

check_uqer_version()


def get_user_traffic():
    principal_name = os.environ['DatayesPrincipalName']
    user_traffic = {}
    try:
        server = __getConn__()
        url = 'https://{}:{}/data/v1/api/getUserTraffic?user={}'.format(server[2], server[3], principal_name)
        response = requests.get(url=url, timeout=5)
        if response.status_code != 200:
            raise Exception(response.text)

        response = response.json()
        if response['success'] is False:
            raise Exception(response)

        for item in response['data']:
            if item['prd_src'] == 'mercury_sdk':
                user_traffic['bytes_limit'] = item['bytes_limit']
                user_traffic['bytes_current'] = item['bytes_current']
                break

        if len(user_traffic) == 0:
            # 异常情况先置空
            user_traffic['bytes_limit'] = 0
            user_traffic['bytes_current'] = 0

        return user_traffic
    except Exception as e:
        raise e


try:
    DataAPI.api_base.replace_api_files()
    for e in list(sys.modules.keys()):
        if e.startswith('DataAPI') or e.startswith('uqer.DataAPI'):
            del sys.modules[e]
    from . import DataAPI
    DataAPI.get_user_traffic = get_user_traffic
except:
    import traceback
    format_print(traceback.format_exc(), with_date=True)
    format_print('upgrade fail.', with_date=True)



# -*- coding: utf-8 -*-
# 通联数据机密
# --------------------------------------------------------------------
# 通联数据股份公司版权所有 © 2013-2022
#
# 注意：本文所载所有信息均属于通联数据股份公司资产。本文所包含的知识和技术概念均属于
# 通联数据产权，并可能由中国、美国和其他国家专利或申请中的专利所覆盖，并受商业秘密或
# 版权法保护。
# 除非事先获得通联数据股份公司书面许可，严禁传播文中信息或复制本材料。
#
# DataYes CONFIDENTIAL
# --------------------------------------------------------------------
# Copyright © 2013-2022 DataYes, All Rights Reserved.
#
# NOTICE: All information contained herein is the property of DataYes
# Incorporated. The intellectual and technical concepts contained herein are
# proprietary to DataYes Incorporated, and may be covered by China, U.S. and
# Other Countries Patents, patents in process, and are protected by trade
# secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from DataYes.
from . import api_base

try:
    from StringIO import StringIO
except:
    from io import StringIO
import pandas as pd
import sys
from datetime import datetime
from .api_base import get_cache_key, get_data_from_cache, put_data_in_cache, pretty_traceback
import inspect

try:
    unicode
except:
    unicode = str

__doc__ = "知识图谱"


def ProThemeGet(theProID="", theProName="", productID="", field="", pandas="1"):
    """
    存储主题产业链各节点的位置及产品代码。

    :param theProID: 节点代码 可以是列表 可空
    :param theProName: 节点名称 可以是列表 可空
    :param productID: 产品代码 可以是列表 可空
    :param field: 所需字段 可以是列表 可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv 可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """

    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()
    requestString = []
    requestString.append('/api/kGraph/getProTheme.csv?ispandas=1&')
    requestString.append("theProID=")
    if hasattr(theProID, '__iter__') and not isinstance(theProID, str):
        if len(theProID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = theProID
            requestString.append(None)
        else:
            requestString.append(','.join([str(item) if not isinstance(item, unicode) else item for item in theProID]))
    else:
        requestString.append(str(theProID) if not isinstance(theProID, unicode) else theProID)
    requestString.append("&theProName=")
    if hasattr(theProName, '__iter__') and not isinstance(theProName, str):
        if len(theProName) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = theProName
            requestString.append(None)
        else:
            requestString.append(
                ','.join([str(item) if not isinstance(item, unicode) else item for item in theProName]))
    else:
        requestString.append(str(theProName) if not isinstance(theProName, unicode) else theProName)
    requestString.append("&productID=")
    if hasattr(productID, '__iter__') and not isinstance(productID, str):
        if len(productID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = productID
            requestString.append(None)
        else:
            requestString.append(
                ','.join([str(item) if not isinstance(item, unicode) else item for item in productID]))
    else:
        requestString.append(str(productID) if not isinstance(productID, unicode) else productID)
    requestString.append("&field=")
    if hasattr(field, '__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join([str(item) if not isinstance(item, unicode) else item for item in field]))
    else:
        requestString.append(str(field) if not isinstance(field, unicode) else field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (
                csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'ProThemeGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join([str(it) if not isinstance(it, unicode) else it for it in item])
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (
                    temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'ProThemeGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n') + 1:])
        csvString = ''.join(csvString)

    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'theProID', u'productID', u'theProName', u'parentID', u'type', u'class', u'position', u'version',
                     u'isNew', u'updateTime']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO,
                              dtype={'theProID': 'str', 'productID': 'str', 'theProName': 'str', 'parentID': 'str',
                                     'type': 'str', 'position': 'str'}, )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()


def ProThemeChainGet(productID="", relProductID="", field="", pandas="1"):
    """
    存储主题产业链中涉及的产业链上下游关系。

    :param productID: 主产品代码，可通过getProTheme获取productID 可以是列表,productID、relProductID至少选择一个
    :param relProductID: 关联产品代码，可通过getProTheme获取productID 可以是列表,productID、relProductID至少选择一个
    :param field: 所需字段 可以是列表 可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv 可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """

    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()
    requestString = []
    requestString.append('/api/kGraph/getProThemeChain.csv?ispandas=1&')
    requestString.append("productID=")
    if hasattr(productID, '__iter__') and not isinstance(productID, str):
        if len(productID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = productID
            requestString.append(None)
        else:
            requestString.append(
                ','.join([str(item) if not isinstance(item, unicode) else item for item in productID]))
    else:
        requestString.append(str(productID) if not isinstance(productID, unicode) else productID)
    requestString.append("&relProductID=")
    if hasattr(relProductID, '__iter__') and not isinstance(relProductID, str):
        if len(relProductID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = relProductID
            requestString.append(None)
        else:
            requestString.append(
                ','.join([str(item) if not isinstance(item, unicode) else item for item in relProductID]))
    else:
        requestString.append(str(relProductID) if not isinstance(relProductID, unicode) else relProductID)
    requestString.append("&field=")
    if hasattr(field, '__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join([str(item) if not isinstance(item, unicode) else item for item in field]))
    else:
        requestString.append(str(field) if not isinstance(field, unicode) else field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (
                csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'ProThemeChainGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join([str(it) if not isinstance(it, unicode) else it for it in item])
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (
                    temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'ProThemeChainGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n') + 1:])
        csvString = ''.join(csvString)

    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'productID', u'relProductID', u'priRelation', u'relativePosition', u'version', u'isNew',
                     u'updateTime']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype={'productID': 'str', 'relProductID': 'str', 'priRelation': 'str',
                                           'relativePosition': 'str'}, )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()


def ProThemeComGet(partyID="", partyShortName="", partyFullName="", productID="", field="", pandas="1"):
    """
    存储主题产业链中涉及的公司产品关系。

    :param partyID: 公司ID，可通过DataAPI.PartyIDGet获取 可以是列表,partyID、partyShortName、partyFullName、productID至少选择一个
    :param partyShortName: 公司简称,partyID、partyShortName、partyFullName、productID至少选择一个
    :param partyFullName: 公司全称，支持模糊查询,partyID、partyShortName、partyFullName、productID至少选择一个
    :param productID: 标准产品代码，可通过getProTheme获取productID 可以是列表,partyID、partyShortName、partyFullName、productID至少选择一个
    :param field: 所需字段 可以是列表 可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv 可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """

    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()
    requestString = []
    requestString.append('/api/kGraph/getProThemeCom.csv?ispandas=1&')
    requestString.append("partyID=")
    if hasattr(partyID, '__iter__') and not isinstance(partyID, str):
        if len(partyID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = partyID
            requestString.append(None)
        else:
            requestString.append(','.join([str(item) if not isinstance(item, unicode) else item for item in partyID]))
    else:
        requestString.append(str(partyID) if not isinstance(partyID, unicode) else partyID)
    if not isinstance(partyShortName, str) and not isinstance(partyShortName, unicode):
        partyShortName = str(partyShortName)

    requestString.append("&partyShortName=%s" % (partyShortName))
    if not isinstance(partyFullName, str) and not isinstance(partyFullName, unicode):
        partyFullName = str(partyFullName)

    requestString.append("&partyFullName=%s" % (partyFullName))
    requestString.append("&productID=")
    if hasattr(productID, '__iter__') and not isinstance(productID, str):
        if len(productID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = productID
            requestString.append(None)
        else:
            requestString.append(
                ','.join([str(item) if not isinstance(item, unicode) else item for item in productID]))
    else:
        requestString.append(str(productID) if not isinstance(productID, unicode) else productID)
    requestString.append("&field=")
    if hasattr(field, '__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join([str(item) if not isinstance(item, unicode) else item for item in field]))
    else:
        requestString.append(str(field) if not isinstance(field, unicode) else field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (
                csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'ProThemeComGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join([str(it) if not isinstance(it, unicode) else it for it in item])
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (
                    temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'ProThemeComGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n') + 1:])
        csvString = ''.join(csvString)

    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'partyID', u'partyShortName', u'partyFullName', u'relationPro', u'productID', u'version',
                     u'isNew', u'updateTime']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype={'partyShortName': 'str', 'partyFullName': 'str', 'relationPro': 'str',
                                           'productID': 'str'}, )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()


def ProThemeIndexDataGet(indicID="", periodDate="", beginDate="", endDate="", field="", pandas="1"):
    """
    存储主题产业链中涉及的指标数据。

    :param indicID: 指标代码，可通过getProThemeIndexInfo获取 可以是列表,indicID、periodDate至少选择一个
    :param periodDate: 数据日期 可以是列表,indicID、periodDate至少选择一个
    :param beginDate: 数据查询起始日期，输入格式"YYYYMMDD" 可空
    :param endDate: 数据查询截至日期，输入格式"YYYYMMDD" 可空
    :param field: 所需字段 可以是列表 可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv 可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """

    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()
    requestString = []
    requestString.append('/api/kGraph/getProThemeIndexData.csv?ispandas=1&')
    requestString.append("indicID=")
    if hasattr(indicID, '__iter__') and not isinstance(indicID, str):
        if len(indicID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = indicID
            requestString.append(None)
        else:
            requestString.append(','.join([str(item) if not isinstance(item, unicode) else item for item in indicID]))
    else:
        requestString.append(str(indicID) if not isinstance(indicID, unicode) else indicID)
    try:
        if isinstance(periodDate, datetime):
            periodDate = periodDate.strftime('%Y%m%d')
        elif isinstance(periodDate, list):
            tds = []
            for td in periodDate:
                td = td.strftime('%Y%m%d')
                tds.append(td)
            periodDate = tds
        else:
            raise Exception
    except:
        if isinstance(periodDate, (str, unicode)):
            tradeDates = periodDate.split(',')
            tds = []
            for td in tradeDates:
                td = td.replace('-', '')
                tds.append(td)
            periodDate = tds
        elif isinstance(periodDate, list):
            tds = []
            for td in periodDate:
                td = td.replace('-', '')
                tds.append(td)
            periodDate = tds

    requestString.append("&periodDate=")
    if hasattr(periodDate, '__iter__') and not isinstance(periodDate, str):
        if len(periodDate) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = periodDate
            requestString.append(None)
        else:
            requestString.append(
                ','.join([str(item) if not isinstance(item, unicode) else item for item in periodDate]))
    else:
        requestString.append(str(periodDate) if not isinstance(periodDate, unicode) else periodDate)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s" % (beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s" % (endDate))
    requestString.append("&field=")
    if hasattr(field, '__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join([str(item) if not isinstance(item, unicode) else item for item in field]))
    else:
        requestString.append(str(field) if not isinstance(field, unicode) else field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (
                csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'ProThemeIndexDataGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join([str(it) if not isinstance(it, unicode) else it for it in item])
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (
                    temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'ProThemeIndexDataGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n') + 1:])
        csvString = ''.join(csvString)

    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'indicID', u'nameCN', u'publishDate', u'periodDate', u'dataValue', u'updateTime']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype={'indicID': 'str', 'nameCN': 'str'}, )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()


def ProThemeIndexInfoGet(productID="", indicID="", nameCN="", field="", pandas="1"):
    """
    存储主题产业链中涉及的指标信息。

    :param productID: 产品代码，可通过getProTheme获取productID 可以是列表,productID、indicID、nameCN至少选择一个
    :param indicID: 指标代码 可以是列表,productID、indicID、nameCN至少选择一个
    :param nameCN: 指标名称，支持模糊查询,productID、indicID、nameCN至少选择一个
    :param field: 所需字段 可以是列表 可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv 可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """

    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()
    requestString = []
    requestString.append('/api/kGraph/getProThemeIndexInfo.csv?ispandas=1&')
    requestString.append("productID=")
    if hasattr(productID, '__iter__') and not isinstance(productID, str):
        if len(productID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = productID
            requestString.append(None)
        else:
            requestString.append(
                ','.join([str(item) if not isinstance(item, unicode) else item for item in productID]))
    else:
        requestString.append(str(productID) if not isinstance(productID, unicode) else productID)
    requestString.append("&indicID=")
    if hasattr(indicID, '__iter__') and not isinstance(indicID, str):
        if len(indicID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = indicID
            requestString.append(None)
        else:
            requestString.append(','.join([str(item) if not isinstance(item, unicode) else item for item in indicID]))
    else:
        requestString.append(str(indicID) if not isinstance(indicID, unicode) else indicID)
    if not isinstance(nameCN, str) and not isinstance(nameCN, unicode):
        nameCN = str(nameCN)

    requestString.append("&nameCN=%s" % (nameCN))
    requestString.append("&field=")
    if hasattr(field, '__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join([str(item) if not isinstance(item, unicode) else item for item in field]))
    else:
        requestString.append(str(field) if not isinstance(field, unicode) else field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (
                csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, 'ProThemeIndexInfoGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join([str(it) if not isinstance(it, unicode) else it for it in item])
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (
                    temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, 'ProThemeIndexInfoGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n') + 1:])
        csvString = ''.join(csvString)

    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'productID', u'indicID', u'nameCN', u'unit', u'freq', u'stat', u'source', u'updateTime']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype={'productID': 'str', 'indicID': 'str', 'nameCN': 'str', 'unit': 'str',
                                           'freq': 'str', 'stat': 'str', 'source': 'str'}, )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()



# -*- coding: utf-8 -*-
# 通联数据机密
# --------------------------------------------------------------------
# 通联数据股份公司版权所有 © 2013-2022
#
# 注意：本文所载所有信息均属于通联数据股份公司资产。本文所包含的知识和技术概念均属于
# 通联数据产权，并可能由中国、美国和其他国家专利或申请中的专利所覆盖，并受商业秘密或
# 版权法保护。
# 除非事先获得通联数据股份公司书面许可，严禁传播文中信息或复制本材料。
#
# DataYes CONFIDENTIAL
# --------------------------------------------------------------------
# Copyright © 2013-2022 DataYes, All Rights Reserved.
#
# NOTICE: All information contained herein is the property of DataYes
# Incorporated. The intellectual and technical concepts contained herein are
# proprietary to DataYes Incorporated, and may be covered by China, U.S. and
# Other Countries Patents, patents in process, and are protected by trade
# secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from DataYes.
RM_STYLE_MAPPING = {"SIZENL": u"非线性市值",
                    "MOMENTUM": u"动量",
                    "SIZE": u"市值",
                    "EARNYILD": u"盈利能力",
                    "LIQUIDTY": u"流动性",
                    "GROWTH": u"成长性",
                    "LEVERAGE": u"杠杆率",
                    "RESVOL": u"波动率",
                    "BTOP": u"价值",
                    "specific_return": u"特殊性成分",
                    "specific_risk": u"特殊性成分",
                    "BETA": u"beta",
                    "INDUSTRY": u"行业",
                    "COUNTRY": u"国家", }

RISK_INDUSTRY_MAP = {
    'dy_cne5_old': 'sw14',
    'dy_cne5_new': 'sw21'
}
INDUSTRY_VERSION_MAP = {
    'sw14':['AERODEF', 'AgriForest', 'Auto', 'BETA', 'BTOP', 'Bank',
            'BuildDeco', 'CHEM', 'CONMAT', 'COUNTRY', 'CommeTrade',
            'Computer', 'Conglomerates', 'EARNYILD', 'ELECEQP',
            'Electronics', 'FoodBever', 'GROWTH', 'Health', 'HouseApp',
            'IronSteel', 'LEVERAGE', 'LIQUIDTY', 'LeiService', 'LightIndus',
            'MOMENTUM', 'MachiEquip', 'Media', 'Mining', 'NonBankFinan',
            'NonFerMetal', 'RESVOL', 'RealEstate', 'SIZE', 'SIZENL',
            'Telecom', 'Textile', 'Transportation', 'Utilities'],
    'sw21':['Agriculture', 'Automobiles', 'BETA', 'BTOP', 'Banks', 'BasicChemicals', 'BeautyCare',
            'BuildMater', 'Chemicals', 'Coal', 'COUNTRY', 'Commerce',
            'Computers', 'Conglomerates', 'ConstrDecor', 'Defense', 'EARNYILD',
            'ElectricalEquip', 'Electronics', 'EnvironProtect', 'FoodBeverages', 'GROWTH', 'HealthCare', 'HomeAppliances',
            'LEVERAGE', 'LIQUIDTY', 'Leisure', 'LightIndustry',
            'MOMENTUM', 'MachineEquip', 'Media', 'Mining', 'NonbankFinan',
            'NonferrousMetals', 'Petroleum', 'PowerEquip', 'RESVOL', 'RealEstate', 'RetailTrade', 'SIZE', 'SIZENL',
            'SocialServices', 'Steel', 'Telecoms', 'TextileGarment', 'TextileApparel', 'Transportation', 'Utilities']
}

INDUSTRY_SW14_TO_21 = {
    'AgriForest': 'Agriculture',
    'Auto': 'Automobiles',
    'Bank': 'Banks',
    'BuildDeco': 'ConstrDecor',
    'CHEM': 'Chemicals',
    'CommeTrade': 'Commerce',
    'Computer': 'Computers',
    'CONMAT': 'BuildMater',
    'AERODEF': 'Defense',
    'ELECEQP': 'ElectricalEquip',
    'FoodBever': 'FoodBeverages',
    'Health': 'HealthCare',
    'HouseApp': 'HomeAppliances',
    'LeiService': 'Leisure',
    'LightIndus': 'LightIndustry',
    'MachiEquip': 'MachineEquip',
    'NonFerMetal': 'NonferrousMetals',
    'NonBankFinan': 'NonbankFinan',
    'IronSteel': 'Steel',
    'Telecom': 'Telecoms',
    'Textile': 'TextileGarment'
}